/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */

package org.enhydra.kelp;

/**
 * Class containing the XMLC version number.
 */
public class KelpVersion {
    public static final String VERSION = "1.2b3";
    public static final String  XMLC_RUN_CLASS =
	"org.enhydra.xml.xmlc.commands.xmlc.XMLC";

    private static final String SERVER_RUN_CLASS30 =
	"com.lutris.multiServer.MultiServer";
    private static final String SERVER_RUN_CLASS40 =
	"org.enhydra.multiServer.bootstrap.Bootstrap";
    private static final String XMLC_CLASS40 =
	"org.enhydra.xml.xmlc.metadata.MetaData";

    public static String getEnhydraVersion() {
      String version = "3.0";
      try {
        Class.forName(KelpVersion.SERVER_RUN_CLASS30);
      } catch (ClassNotFoundException e) {
        version = "4.0";
      }
      return version;
    }

    public static boolean isXMLCInClassPath() {
      boolean inPath = true;
      try {
 	 Class testClass = Class.forName(KelpVersion.XMLC_RUN_CLASS);
      } catch (java.lang.ClassNotFoundException e) {
         inPath = false;
      }
      return inPath;
    }

    public static boolean isEnhydraInClassPath() {
      boolean inPath = true;
      String checkClass = XMLC_CLASS40;
      if (KelpVersion.getEnhydraVersion().startsWith("3")) {
         checkClass = SERVER_RUN_CLASS30;
      }
      try {
 	 Class testClass = Class.forName(checkClass);
      } catch (java.lang.ClassNotFoundException e) {
         inPath = false;
      }
      return inPath;
    }


    /**
     * Get the class to run to start MultiServer.
     */
    public static String getMultiServerRunClass() {
      String runClass = KelpVersion.SERVER_RUN_CLASS30;
      try {
        Class.forName(runClass);
      } catch (ClassNotFoundException e) {
        runClass = KelpVersion.SERVER_RUN_CLASS40;
      }
      return runClass;
    }

    /**
     * Prevent instantiation.
     */
    private KelpVersion() {
    }

}

