/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */

package org.enhydra.kelp.common.bridge;

// XMLC imports
import org.enhydra.xml.xmlc.compiler.Parse;
import org.enhydra.xml.xmlc.dom.XMLCDocument;
import org.enhydra.xml.xmlc.XMLCException;

// Standard imports
import java.io.PrintWriter;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;

public class ParserV3 implements Parser {
  private Parse parseV3 = null;
  private PrintWriter traceWriter = null;
  private PrintWriter infoWriter = null;
  private MetaDataHandler handler = null;

  public ParserV3(PrintWriter trace, PrintWriter info) {
    traceWriter = trace;
    infoWriter = info;

    Class parseClass = null;
    Constructor[] cons = new Constructor[0];
    Object[] values = new Object[2];
    try {
      parseClass = Class.forName("org.enhydra.xml.xmlc.compiler.Parse");
      cons = parseClass.getConstructors();
      for (int i = 0 ; i < cons.length ; i++ ) {
        if (cons[i].getParameterTypes().length == 2) {
          values[0] = createErrorReporter(traceWriter);
          values[1] = infoWriter;
          parseV3 = (Parse) cons[i].newInstance(values);
          break;
        }
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  public MetaDataHandler getMetaDataHandler() {
    return handler;
  }

  public void setMetaDataHandler(MetaDataHandler h) {
    handler = h;
  }

  public PrintWriter getTraceWriter() {
    return traceWriter;
  }

  public PrintWriter getInfoWriter() {
    return infoWriter;
  }

  public XMLCDocument parse() throws XMLCException {
    XMLCDocument doc = null;
    doc = (XMLCDocument) callParse();
    return doc;
  }

  private Object createErrorReporter(PrintWriter writer) {
    Object reporter = null;
    Constructor con = null;
    Class reportClass = null;
    Class[] types = new Class[1];
    Object[] values = new Object[1];
    types[0] = writer.getClass();
    try {
      reportClass = Class.forName("org.enhydra.xml.xmlc.compiler.ErrorReporter");
      con = reportClass.getConstructor(types);
      values[0] = writer;
      reporter = con.newInstance(values);
    } catch (Exception e) {
      e.printStackTrace();
    }
    return reporter;
  }

    private Object callParse()
         throws XMLCException {
      Object doc = null;
      Method meths[] = parseV3.getClass().getDeclaredMethods();
      Method parse = null;
      for (int i = 0 ; i < meths.length ; i++) {
        if (meths[i].getName().equals("parse")) {
          parse = meths[i];
          break;
        }
      }
      if (parse != null) {
        Object params[] = new Object[1];
        params[0] = handler.getMetaData();
        try {
          doc = parse.invoke(parseV3, params);
        } catch (java.lang.IllegalAccessException e) {
          e.printStackTrace();
        } catch (java.lang.reflect.InvocationTargetException e) {
          if (e.getTargetException() instanceof XMLCException) {
            throw (XMLCException) e.getTargetException();
          } else {
            XMLCException xe = new XMLCException(e.getTargetException().toString());
            e.getTargetException().printStackTrace();
            throw xe;
          }
        }
      }
      return doc;
    }

}
