/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */
package org.enhydra.kelp.common.node;

// Kelp imports
import org.enhydra.kelp.common.PropUtil;
import org.enhydra.kelp.KelpVersion;

// Standard imports
import java.io.File;
import java.util.Vector;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
abstract public class OtterProject implements OtterNode {
    private final String    XMLC_MAKE = "enhydra.xmlc.make";
    private final String    XMLC_DUMP = "enhydra.xmlc.dump";
    private final String    XMLC_PARSEINFO = "enhydra.xmlc.parseInfo";
    private final String    XMLC_PRINTINFO = "enhydra.xmlc.printInfo";
    private final String    XMLC_VERBOSE = "enhydra.xmlc.verbose";
    private final String    XMLC_PRINTACCESSMETHODS =
        "enhydra.xmlc.printAccessMethods";
    private final String    XMLC_SAVE_OUTPUT =
        "enhydra.xmlc.saveOutputToFile";
    private final String    XMLC_OUTPUT_FILENAME =
        "enhydra.xmlc.outputToFilename";
    private final String    XMLC_MAP_PACKAGE_FROM =
        "enhydra.xmlc.map.package.fromFolder";
    private final String    XMLC_MAP_PACKAGE_TO =
        "enhydra.xmlc.map.package.toPackage";
    private final String    XMLC_MAP_PACKAGE_LENGTH =
        "enhydra.xmlc.map.package.length";
    private final String    XMLC_MAP_SCOPE = "enhydra.xmlc.map.scope";
    private final String    XMLC_RESOURCE_MAP = "enhydra.xmlc.resource.map";
    private final String    XMLC_RESOURCE_PATH = "enhydra.xmlc.resource.path";
    private final String    CONF_MAKE = "enhydra.conf.make";
    private final String    CONF_OVERWRITE = "enhydra.conf.overwrite";
    private final String    CONF_PATH = "enhydra.conf.path";
    private final String    CONF_TABLE_LENGTH = "enhydra.conf.table.length";
    private final String    CONF_TABLE_FIND = "enhydra.conf.table.find";
    private final String    CONF_TABLE_REPLACE = "enhydra.conf.table.replace";
    private final String    CONF_STARTUP = "enhydra.conf.startup";
    private final String    CONF_STARTUP_JAVA = "enhydra.conf.startup.java";
    private final String    CONF_STARTUP_RUN = "enhydra.conf.startup.run";
    private final String    CONF_STARTUP_CONF = "enhydra.conf.startup.conf";
    public final static int MAP_SCOPE_ALL = 0;
    public final static int MAP_SCOPE_SELECTED = 1;
    public final static int MAP_SCOPE_NONE = 2;


    abstract public String getCodeGenDefaultDestination();

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public OtterTextFileNode getFirstPolicy();

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public OtterTextFileNode[] getAllDeploymentDescs();

    /**
     * Method declaration
     *
     *
     * @param extensions
     *
     * @return
     *
     * @see
     */
    abstract public OtterTextFileNode[] getAllText(String[] extensions);

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public OtterImageFileNode[] getAllImages();

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public OtterHTMLFileNode[] getAllHTMLFiles();

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public OtterWMLFileNode[] getAllWMLFiles();

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public Object getNativeProject();

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public String getName();

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public boolean isOpenBuild();

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public String getClassOutputPath();
    abstract public void setClassOutputPath(String p);

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public String getSourcePath();

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public String[] getSourcePathArray();

    /**
     * Method declaration
     *
     *
     * @param pathAsArray
     *
     * @see
     */
    abstract public void setSourcePathArray(String[] pathAsArray);

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public String getClassPath();

    /**
     * Method declaration
     *
     *
     * @param property
     *
     * @return
     *
     * @see
     */
    abstract public String getProperty(String property);

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public OtterNodeFactory getNodeFactory();

    /**
     * Method declaration
     *
     *
     * @see
     */
    abstract public void configureRunClass();

    /**
     * Method declaration
     *
     *
     * @param property
     * @param value
     *
     * @see
     */
    abstract public void setProperty(String property, String value);

    /**
     * Method declaration
     *
     *
     * @param property
     * @param value
     *
     * @see
     */
    abstract public void setProperty(String property, int value);

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public OtterXMLCNode[] getAllXMLCNodes() {
        OtterHTMLFileNode[] htmlNodes = new OtterHTMLFileNode[0];
        OtterWMLFileNode[]  wmlNodes = new OtterWMLFileNode[0];
        OtterXMLCNode[]     xmlcNodes = new OtterXMLCNode[0];
        htmlNodes = getAllHTMLFiles();
        wmlNodes = getAllWMLFiles();
        xmlcNodes = new OtterXMLCNode[htmlNodes.length + wmlNodes.length];
        for (int i = 0; i < htmlNodes.length; i++) {
            xmlcNodes[i] = new OtterXMLCNode(htmlNodes[i]);
        }
        for (int i = 0; i < wmlNodes.length; i++) {
            xmlcNodes[htmlNodes.length + i] = new OtterXMLCNode(wmlNodes[i]);
        }
        return xmlcNodes;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getXMLCOptionFilename() {
        String filename = new String();
        File   file = PropUtil.getFileRelativeToProject(this,
                OtterNode.XMLC_OPTION_FILENAME);

        if ((file != null) && file.exists() && (!file.isDirectory())) {
            filename = file.getAbsolutePath();
        }
        return filename;
    }

    /**
     * Method declaration
     *
     *
     * @param filename
     *
     * @see
     */
    public void setXMLCOptionFilename(String filename) {
        PropUtil.putFileRelativeToProject(this,
                                          OtterNode.XMLC_OPTION_FILENAME,
                                          filename);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getXMLCParameters() {
        return getProperty(OtterNode.XMLC_PARAMETERS);
    }

    /**
     * Method declaration
     *
     *
     * @param p
     *
     * @see
     */
    public void setXMLCParameters(String p) {
        setProperty(OtterNode.XMLC_PARAMETERS, p);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isDumpSelected() {
        return readBooleanProperty(XMLC_DUMP, false);
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setDumpSelected(boolean b) {
        writeBooleanProperty(XMLC_DUMP, b);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isXMLCMake() {
        return readBooleanProperty(XMLC_MAKE, false);
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setXMLCMake(boolean b) {
        writeBooleanProperty(XMLC_MAKE, b);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isResourceMap() {
        return readBooleanProperty(XMLC_RESOURCE_MAP, true);
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setResourceMap(boolean b) {
        writeBooleanProperty(XMLC_RESOURCE_MAP, b);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getResourcePath() {
        String out = new String();

        if (!isResourceMap()) {
            out = getProperty(XMLC_RESOURCE_PATH);
            out = PropUtil.expandPathRelativeToProject(this, out);
        }
        return out;
    }

    /**
     * Method declaration
     *
     *
     * @param path
     *
     * @see
     */
    public void setResourcePath(String path) {
        path = PropUtil.compressPathRelativeToProject(this, path);
        setProperty(XMLC_RESOURCE_PATH, path);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isParseInfoSelected() {
        return readBooleanProperty(XMLC_PARSEINFO, false);
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setParseInfoSelected(boolean b) {
        writeBooleanProperty(XMLC_PARSEINFO, b);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isPrintAccessMethodsSelected() {
        return readBooleanProperty(XMLC_PRINTACCESSMETHODS, false);
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setPrintAccessMethodsSelected(boolean b) {
        writeBooleanProperty(XMLC_PRINTACCESSMETHODS, b);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isPrintInfoSelected() {
        return readBooleanProperty(XMLC_PRINTINFO, false);
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setPrintInfoSelected(boolean b) {
        writeBooleanProperty(XMLC_PRINTINFO, b);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isVerboseSelected() {
        return readBooleanProperty(XMLC_VERBOSE, false);
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setVerboseSelected(boolean b) {
        writeBooleanProperty(XMLC_VERBOSE, b);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isSaveOutputSelected() {
        return readBooleanProperty(XMLC_SAVE_OUTPUT, true);
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setSaveOutputSelected(boolean b) {
        writeBooleanProperty(XMLC_SAVE_OUTPUT, b);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public int getMapScope() {
        String strScope;
        int    intScope;

        strScope = getProperty(XMLC_MAP_SCOPE);
        intScope = PropUtil.stringToInt(strScope, MAP_SCOPE_ALL);
        return intScope;
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setMapScope(int i) {
        setProperty(XMLC_MAP_SCOPE, i);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getOutputFilename() {
        String out = null;

        out = getProperty(XMLC_OUTPUT_FILENAME);
        return out;
    }

    /**
     * Method declaration
     *
     *
     * @param n
     *
     * @see
     */
    public void setOutputFilename(String n) {
        setProperty(XMLC_OUTPUT_FILENAME, n);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String[][] getPackageMap() {
        String[][] map = PropUtil.getArrayProperty(this,
                                                   XMLC_MAP_PACKAGE_LENGTH,
                                                   XMLC_MAP_PACKAGE_FROM,
                                                   XMLC_MAP_PACKAGE_TO);
        File       from;

        for (int i = 0; i < map.length; i++) {
            map[i][0] = PropUtil.expandPathRelativeToProject(this, map[i][0]);
        }
        return map;
    }

    /**
     * Method declaration
     *
     *
     * @param map
     *
     * @see
     */
    public void setPackageMap(String[][] map) {
        for (int i = 0; i < map.length; i++) {
            map[i][0] = PropUtil.compressPathRelativeToProject(this,
                    map[i][0]);
        }
        PropUtil.putArrayProperty(this, XMLC_MAP_PACKAGE_LENGTH,
                                  XMLC_MAP_PACKAGE_FROM, XMLC_MAP_PACKAGE_TO,
                                  map);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String[][] getDefaultConfTable() {
        File       defaultConf = new File(getServletConfPath());
        String[][] defaultTable = {
             {
                "@CLASSES@", getClassOutputPath()
            }, {
                "@SERVLET_CONF_DIR@", defaultConf.getAbsolutePath()
            }
        };

        for (int i = 0; i < defaultTable.length; i++) {
            defaultTable[i][1] = PropUtil.compressPathRelativeToProject(this,
                    defaultTable[i][1]);
        }
        return defaultTable;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String[][] getConfTable() {
        String[][] confTable = PropUtil.getArrayProperty(this,
                CONF_TABLE_LENGTH, CONF_TABLE_FIND, CONF_TABLE_REPLACE);

        // replace relative paths in replaceWith column
        if (confTable.length == 0) {
            confTable = getDefaultConfTable();
        } else {
            for (int i = 0; i < confTable.length; i++) {
                confTable[i][1] = PropUtil.compressPathRelativeToProject(this,
                        confTable[i][1]);
            }
        }
        return confTable;
    }

    /**
     * Method declaration
     *
     *
     * @param map
     *
     * @see
     */
    public void setConfTable(String[][] confTable) {

        // replace relative paths in replaceWith column
        for (int i = 0; i < confTable.length; i++) {
            confTable[i][1] = PropUtil.compressPathRelativeToProject(this,
                    confTable[i][1]);
        }
        PropUtil.putArrayProperty(this, CONF_TABLE_LENGTH, CONF_TABLE_FIND,
                                  CONF_TABLE_REPLACE, confTable);
    }

    /**
     * Method declaration
     *
     *
     * @param property
     * @param def
     *
     * @return
     *
     * @see
     */
    private boolean readBooleanProperty(String property, boolean def) {
        boolean flag = def;
        String  in = getProperty(property);

        return PropUtil.stringToBoolean(in, def);
    }

    /**
     * Method declaration
     *
     *
     * @param property
     * @param b
     *
     * @see
     */
    private void writeBooleanProperty(String property, boolean b) {
        String flag = b ? "true" : "false";

        setProperty(property, flag);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getGenerateToDirectory() {
        String genToDir = null;

        if (readBooleanProperty("GenerateSourceToOutDir", true)) {
            genToDir = getClassOutputPath() + File.separator
                       + "Generated Source";
        }
        return genToDir;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isConfOverwriteSelected() {
        String in = getProperty(CONF_OVERWRITE);

        return PropUtil.stringToBoolean(in, false);
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setConfOverwriteSelected(boolean b) {
        setProperty(CONF_OVERWRITE, PropUtil.booleanToString(b));
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getServletConfPath() {
        File         file = new File(getClassOutputPath());
        StringBuffer outPath = new StringBuffer();
        outPath.append(file.getParent());
        if (!outPath.toString().endsWith(File.separator)) {
            outPath.append(File.separator);
        }
        outPath.append("output");
        outPath.append(File.separator);
        outPath.append("servlet");
        file = PropUtil.getFileRelativeToProject(this, CONF_PATH);
        if ((file != null) && (!file.isFile())) {
            outPath.setLength(0);
            outPath.append(file.getAbsolutePath());
        }
        return outPath.toString();
    }

    /**
     * Method declaration
     *
     *
     * @param directory
     *
     * @see
     */
    public void setConfPath(String directory) {
        PropUtil.putFileRelativeToProject(this, CONF_PATH, directory);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isConfStartupSelected() {
        String in = getProperty(CONF_STARTUP);

        return PropUtil.stringToBoolean(in, true);
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setConfStartupSelected(boolean b) {
        setProperty(CONF_STARTUP, PropUtil.booleanToString(b));
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isConfStartupRun() {
        String in = getProperty(CONF_STARTUP_RUN);
        return PropUtil.stringToBoolean(in, true);
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setConfStartupRun(boolean b) {
        setProperty(CONF_STARTUP_RUN, PropUtil.booleanToString(b));
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isConfStartupJava() {
        String in = getProperty(CONF_STARTUP_JAVA);

        return PropUtil.stringToBoolean(in, !isOpenBuild());
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setConfStartupJava(boolean b) {
        setProperty(CONF_STARTUP_JAVA, PropUtil.booleanToString(b));
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getConfStartupConf() {
        String conf = getProperty(CONF_STARTUP_CONF);

        if (conf == null || conf.trim().length() == 0) {

            // set default;
            if (KelpVersion.getEnhydraVersion().startsWith("3")) {
                conf = getServletConfPath() + File.separator + "servlet.conf";
            } else {
                File file = new File(getServletConfPath());

                conf = file.getParent();
            }
            setConfStartupConf(conf);
        } else {
            File file = PropUtil.getFileRelativeToProject(this,
                    CONF_STARTUP_CONF);

            if (file != null) {
                if (KelpVersion.getEnhydraVersion().startsWith("4")
                        && file.isFile()) {
                    file = file.getParentFile();
                }
                conf = file.getAbsolutePath();
            }
        }
        return conf;
    }

    /**
     * Method declaration
     *
     *
     * @param conf
     *
     * @see
     */
    public void setConfStartupConf(String conf) {
        PropUtil.putFileRelativeToProject(this, CONF_STARTUP_CONF, conf);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isConfMake() {
        return readBooleanProperty(CONF_MAKE, false);
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setConfMake(boolean b) {
        writeBooleanProperty(CONF_MAKE, b);
    }

    /**
     * Method declaration
     *
     *
     * @param node
     *
     * @return
     *
     * @see
     */
    public String getSourcePathOf(OtterNode node) {
        String[] path = getSourcePathArray();
        String   sourcePath = new String();
        File     pathDir = null;
        File     nodeFile = new File(node.getFilePath());

        sourcePath = nodeFile.getParent();

        for (int i = 0; i < path.length; i++) {
            pathDir = new File(path[i]);
            if (nodeFile.getAbsolutePath().toLowerCase().startsWith(pathDir.getAbsolutePath().toLowerCase().trim())) {
                sourcePath = pathDir.getAbsolutePath().trim();
                break;
            }
        }
        return sourcePath;
    }

}
