/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */

package org.enhydra.kelp.common.wizard.xmlc;

import java.awt.Component;
import javax.swing.*;
import javax.swing.table.*;

/**
 * Override the default TableCellRenderer so that we can set the tool tip
 * for each cell. The tool tip is handy for long path names.
 */
public class CellRendererWithToolTip extends DefaultTableCellRenderer {

    /**
     * Create a CellRendererToolip.
     */
    public CellRendererWithToolTip() {}

    /**
     * This method is sent to the renderer by the drawing table
     * to configure the renderer appropriately before drawing. Return
     * the Component used for drawing.
     * 
     * @param table
     * The JTable that is asking the renderer to draw. This parameter can be null.
     * @param value
     * The value of the cell to be rendered. It is up to the specific renderer
     * to interpret and draw the value. eg. if value is the String "true",
     * it could be rendered as a string or it could be rendered as a check box
     * that is checked. null is a valid value.
     * @param isSelected
     * If true the cell is to be renderer with selection highlighting
     * @param hasFocus
     * If true the cell has focus.
     * @param row
     * The row index of the cell being drawn. When drawing the header the rowIndex is -1.
     * @param column
     * The column index of the cell being drawn
     */
    public Component getTableCellRendererComponent(JTable table, 
	    Object value, boolean isSelected, boolean hasFocus, int row, 
	    int column) {
	Component c = super.getTableCellRendererComponent(table, value, 
		isSelected, hasFocus, row, column);

	if (c instanceof JLabel) {    // add the tool tip
	    JLabel l = (JLabel) c;

	    l.setToolTipText(l.getText());
	} 

	return c;
    } 

}



/*--- formatting done in "Sun Java Convention" style on 01-16-2000 ---*/

