/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */
package org.enhydra.kelp.jbuilder.build;

import org.enhydra.kelp.common.wizard.conf.ConfBuildThread;
import org.enhydra.kelp.common.capture.WriteEvent;
import org.enhydra.kelp.common.capture.WriteListener;
import org.enhydra.kelp.common.node.OtterTemplateNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.jbuilder.node.PrimeProject;
import com.borland.primetime.build.*;
import com.borland.primetime.node.*;
import com.borland.jbuilder.node.*;
import com.borland.jbuilder.build.*;
import java.util.*;

/**
 * BuildTask implementation for calling XMLC.
 *
 */
public class ConfBuildTask implements BuildTask, WriteListener {
    public static final Object SHARED_KEY = new Object();
    private BuildProcess       buildProcess;
    private Vector	       nodeVector = new Vector();
    private Vector	       writeListeners = new Vector();

    public ConfBuildTask() {}

    /**
     * Copy conf.in to output directory and replace placeholders.
     */
    public boolean build(BuildProcess buildProcess, Project project,
			 boolean rebuild) {
	this.buildProcess = buildProcess;

	OtterProject        otterProject = new PrimeProject((JBProject) project);
	int	    	    nodeCount = nodeVector.size();
        int                 compileCount = nodeCount;
        OtterTemplateNode   current = null;
        ConfBuildThread     buildThread = null;

        // remove upToDate nodes from vector if not a rebuild.
        if (! rebuild) {
   	  for (int i = 0; i < nodeCount; i++) {
	    current = (OtterTemplateNode) nodeVector.elementAt(i);
            if (upToDate(current)) {
               nodeVector.setElementAt(new Object(), i);
               compileCount--;
            }
          }
	}

	OtterTemplateNode[] nodes = new OtterTemplateNode[compileCount];

	for (int i = 0; i < nodeCount; i++) {
            if (nodeVector.elementAt(i) instanceof OtterTemplateNode) {
  	      nodes[i] = (OtterTemplateNode) nodeVector.elementAt(i);
            }
	}

	WriteListener[] w = new WriteListener[1];

	w[0] = this;

        if (otterProject.isConfMake()) {
  	   buildProcess.fireBuildStatus("Processing conf.in files...",
	  			       true);

	   buildThread = new ConfBuildThread(w, true);

	   buildThread.setProject(otterProject);
	   buildThread.setNodes(nodes);
	   buildThread.setPrintProgress(false);
	   buildThread.runInCurrentThread();
        }

	return true;
    }

    /**
     * Build messages.
     *
     */
    public void onWrite(WriteEvent e) {
	if (buildProcess != null) {
	    buildProcess.fireBuildMessage(e.getString(), e.getString());
	}
    }

    /**
     * Collect list of nodes to build.
     *
     */
    public void addNode(OtterTemplateNode n) {
	nodeVector.addElement(n);
    }

 /**
     * Check if node up to date.
     *
     */
    private boolean upToDate(OtterTemplateNode templateNode) {
	boolean built = false;
        FileNode fileNode = (FileNode) templateNode.getNativeNode();
	if (fileNode.isModified()) {
	    try {
		fileNode.save();
	    } catch (Exception e) {
		e.printStackTrace();
	    }
	} else if (!templateNode.getOutputFile().exists()) {}
	else if (templateNode.getOutputFile().lastModified()
		 > fileNode.getUrl().getFileObject().lastModified()) {
	    built = true;
	}

	return built;
    }


}




