/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */
package org.enhydra.kelp.jbuilder.build;

import org.enhydra.kelp.jbuilder.node.PrimeTemplateNode;
import org.enhydra.kelp.common.node.OtterTemplateNode;
import org.enhydra.kelp.jbuilder.properties.ConfProperties;
import org.enhydra.kelp.common.PropUtil;
import com.borland.primetime.node.TextFileNode;
import com.borland.primetime.node.FileNode;
import com.borland.primetime.node.Node;
import com.borland.primetime.node.Project;
import com.borland.primetime.build.Builder;
import com.borland.primetime.build.BuildProcess;
import com.borland.primetime.build.BuilderManager;

/**
 * This class controls building and maintaining properties for HTML/XML files
 * using the Enhydra XMLC Compiler.
 *
 */
public class ConfBuilder implements Builder {
    public static final String GENERATE_CLASS = "Generate";

    public ConfBuilder() {}

    /**
     * Register open tool
     *
     */
    public static void initOpenTool(byte majorVersion, byte minorVersion) {
	BuilderManager.registerBuilder(new ConfBuilder());
    }

    /**
     * Called by the Build System when make or rebuild occurs
     */
    public void updateBuildProcess(BuildProcess buildProcess,
				   Node nativeNode) {
        ConfBuildTask     task      = null;
        FileNode          fileNode  = null;
	OtterTemplateNode templateNode  = null;
        String            stringIn  = null;
        boolean           selected  = false;

	if (nativeNode instanceof FileNode) {
	  fileNode = (FileNode) nativeNode;
	}

	if (fileNode != null &&
           fileNode.getUrl().getFileObject().getAbsolutePath().toLowerCase().endsWith("conf.in")) {
           templateNode = new PrimeTemplateNode((TextFileNode)fileNode);
        }

        if (templateNode != null && templateNode.isSelected()) {
  	  task = (ConfBuildTask) buildProcess.getTask(ConfBuildTask.SHARED_KEY);
	  if (task == null) {
	      task = new ConfBuildTask();
	      // There is both a map and a list
	      buildProcess.putTask(ConfBuildTask.SHARED_KEY, task);
              buildProcess.getBuildTaskList().add(task);
	  }
 	  task.addNode(templateNode);
        }
    }


}





