/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */
package org.enhydra.kelp.jbuilder.build;

import org.enhydra.kelp.KelpVersion;
import org.enhydra.kelp.jbuilder.node.PrimeHTMLFileNode;
import org.enhydra.kelp.common.node.OtterXMLCNode;
import com.borland.jbuilder.node.HTMLFileNode;
import com.borland.primetime.node.Node;
import com.borland.primetime.node.Project;
import com.borland.primetime.build.Builder;
import com.borland.primetime.build.BuildProcess;
import com.borland.primetime.build.BuilderManager;

import java.util.List;

/**
 * This class controls building and maintaining properties for HTML/XML files
 * using the Enhydra XMLC Compiler.
 *
 */
public class XMLCBuilder implements Builder {
    public static final String GENERATE_CLASS = "Generate";

    public XMLCBuilder() {}

    /**
     * Register open tool
     *
     */
    public static void initOpenTool(byte majorVersion, byte minorVersion) {
	if (KelpVersion.isEnhydraInClassPath()) {
   	   BuilderManager.registerBuilder(new XMLCBuilder());
        } else {
           System.out.println("Unable to install XMLC builder: " +
             "Enhydra not found in IDE class path.");
        }
    }

    /**
     * Called by the Build System when make or rebuild occurs
     */
    public void updateBuildProcess(BuildProcess buildProcess,
				   Node nativeNode) {

	// TODO: ?? Do we want to support XMLFileNode ??
	PrimeHTMLFileNode primeNode = null;
	HTMLFileNode      htmlNode = null;
	OtterXMLCNode     xmlcNode = null;

	if (nativeNode instanceof HTMLFileNode) {
	    htmlNode = (HTMLFileNode) nativeNode;
	    primeNode = new PrimeHTMLFileNode(htmlNode);
            if (primeNode.getProject().isXMLCMake()) {

  	      // Check if we want to build this specific HTML file.
	      if (primeNode.isSelected()) {
		  XMLCBuildTask task =
		      (XMLCBuildTask) buildProcess.getTask(XMLCBuildTask.SHARED_KEY);

  		  if (task == null) {
		      task = new XMLCBuildTask();

		      // There is both a map and a list
		      buildProcess.putTask(XMLCBuildTask.SHARED_KEY, task);
                      // Add xmlc task to top of list so genrated DOM
                      // classes are available when dependent classes
                      // are compiled.
                      List list = buildProcess.getBuildTaskList();
                      list.add(0,task);

		  }
	          xmlcNode = new OtterXMLCNode(primeNode);
    		  task.addNode(xmlcNode);
               }
	    }
	}
    }


}





