/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */

package org.enhydra.kelp.jbuilder.node;

// Kelp imports
import org.enhydra.kelp.common.PropUtil;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterWMLFileNode;
import org.enhydra.kelp.common.node.OtterSelectionNode;
import org.enhydra.kelp.common.node.OtterResourceNode;
// JBuilder Foundation imports
import com.borland.primetime.node.FileNode;
import com.borland.primetime.node.TextFileNode;
import com.borland.primetime.node.Project;
import com.borland.primetime.node.Node;
import com.borland.primetime.vfs.Url;
import com.borland.primetime.node.DuplicateNodeException;
// Standard imports
import java.io.File;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
public class PrimeWMLFileNode extends PrimeNode
    implements OtterWMLFileNode {

    private Exception exception = null;

    public PrimeWMLFileNode(TextFileNode n) {
        super(n);
    }

    /**
     * Constructor declaration
     *
     *
     * @param otterProject
     * @param otterNode
     * @param file
     *
     * @see
     */
    public PrimeWMLFileNode(OtterProject otterProject, OtterNode otterNode,
			     String file) {
	Project      nativeProject = null;
	Node	     nativeNode = null;
	TextFileNode nativeTextNode = null;

	if (otterProject.getNativeProject() instanceof Project) {
	    nativeProject = (Project) otterProject.getNativeProject();
	}

	if (otterNode.getNativeNode() instanceof Node) {
	    nativeNode = (Node) otterNode.getNativeNode();
	}

	if (nativeProject != null && nativeNode != null) {
	    try {
		nativeTextNode = new TextFileNode(nativeProject, nativeNode,
						  new Url(new File(file)));
	    } catch (DuplicateNodeException e) {
		System.err.println(e);
		System.err.println(file + " already in project.");
	    }
	}

	setNativeNode(nativeTextNode);
    }



  public boolean isSelected() {
   String  in = getProperty(OtterSelectionNode.SELECTED);
   return PropUtil.stringToBoolean(in, false);
  }

  public void setSelected(boolean b) {
	setProperty(OtterSelectionNode.SELECTED,
	   PropUtil.booleanToString(b));
      if (b) {
        setStatic(false);
      }
  }

  public boolean isStatic() {
   String  in = getProperty(OtterResourceNode.RESOURCE_STATIC);
   return PropUtil.stringToBoolean(in, false);
  }

  public void setStatic(boolean b) {
	setProperty(OtterResourceNode.RESOURCE_STATIC,
	   PropUtil.booleanToString(b));
        if (b) {
          setSelected(false);
        }
  }




    /**
     * Save Exception information for reporting. This lets exceptions from
     * multiple nodes to be reported in a batch.
     *
     * @param e
     * An exception that occured while invoking XMLC.
     */
    public void setException(Exception e) {
	exception = e;
    }

    /**
     * Get an exception that occured when invoking XMLC. Null if the node
     * has not been compiled or was compiled without error.
     */
    public Exception getException() {
	return exception;
    }


  public void preCompile() {
    if (getNativeNode() instanceof TextFileNode) {
       TextFileNode nativeNode = (TextFileNode) getNativeNode();
       if (nativeNode.isModified()) {
         try {
           nativeNode.save();
         } catch (com.borland.primetime.vfs.InvalidUrlException e) {
           e.printStackTrace();
         } catch (java.io.IOException e) {
           e.printStackTrace();
         }

      }
    }
     // nothing extra to do for Primetime.
  }


}

