/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */

package org.enhydra.kelp.jbuilder.properties;

import org.enhydra.kelp.common.swing.SwingUtil;
import com.borland.primetime.ide.Browser;
import com.borland.primetime.node.FileNode;
import com.borland.primetime.node.Node;
import com.borland.primetime.node.Project;
import com.borland.primetime.properties.PropertyPageFactory;
import com.borland.primetime.properties.PropertyPage;
import com.borland.primetime.properties.PropertyManager;
import com.borland.primetime.properties.PropertyGroup;

public class ConfProperties implements PropertyGroup {
  public static final String GENERATE_CLASS = "Generate";

  public ConfProperties() {
  }


  /**
   * Initialize this OpenTool
   */
  public static void initOpenTool(byte majorVersion, byte minorVersion) {
	PropertyManager.registerPropertyGroup(new ConfProperties());
  }

  /**
   * Returns a PropertyPageFactory for the project or for an
   * "conf.in" file node.  Null if topic is something else.
   */
  public PropertyPageFactory getPageFactory(final Object topic) {
    PropertyPageFactory factory = null;
    FileNode fn;
    Project project;
    if (topic instanceof FileNode) {
      fn = (FileNode) topic;
      project = ((Node)topic).getProject();
      if (fn.getUrl().getFileObject().getAbsolutePath().toLowerCase().endsWith("conf.in")
        && project != null) {
        // Node property page to set your Conf properties
        factory = (new PropertyPageFactory("Enhydra Configure") {
          public PropertyPage createPropertyPage() {
            return new ConfNodePropertyPage(topic);
          }
        });
      }
    } else if (topic instanceof Project) {
        // Project property page to set your XMLC properties
        factory = (new PropertyPageFactory("Enhydra Configure") {
          public PropertyPage createPropertyPage() {
            return new ConfProjectPropertyPage(topic);
          }
        });
    }
    return factory;
  }

  public void initializeProperties() {
  }
}
