/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */
package org.enhydra.kelp.jbuilder.properties;

// Kelp imports
import org.enhydra.kelp.common.node.OtterResourceNode;
import org.enhydra.kelp.common.node.OtterXMLCNode;
import org.enhydra.kelp.common.swing.SwingUtil;
import org.enhydra.kelp.jbuilder.node.PrimeHTMLFileNode;
import org.enhydra.kelp.jbuilder.node.PrimeWMLFileNode;

// JBuilder Foundation Imports
import com.borland.jbuilder.node.HTMLFileNode;
import com.borland.primetime.help.HelpTopic;
import com.borland.primetime.help.ZipHelpTopic;
import com.borland.primetime.node.TextFileNode;
import com.borland.primetime.properties.PropertyPage;

// Standard imports
import java.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision$
 */
public class XMLCNodePropertyPage extends PropertyPage {
    private XMLCNodePropertyPanel nodePanel = new XMLCNodePropertyPanel();
    private BorderLayout          layoutMain = new BorderLayout();
    private final String          HELP_PAGE =
        "file:../kelp1.2/doc/developer/jb-kelp.html#xmlcnode";

    /**
     * Constructor declaration
     *
     *
     * @param topic
     *
     * @see
     */
    public XMLCNodePropertyPage(Object topic) {
        if (topic instanceof TextFileNode) {
            OtterXMLCNode     xmlcNode = null;
            OtterResourceNode resourceNode = null;
            TextFileNode      nativeText = (TextFileNode) topic;

            if (nativeText instanceof HTMLFileNode) {
                resourceNode =
                    new PrimeHTMLFileNode((HTMLFileNode) nativeText);
            } else {
                resourceNode = new PrimeWMLFileNode(nativeText);
            }
            xmlcNode = new OtterXMLCNode(resourceNode);
            try {
                nodePanel.setNode(xmlcNode);
                jbInit();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    public void readProperties() {
        nodePanel.readProperties();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public HelpTopic getHelpTopic() {
        ZipHelpTopic t = new ZipHelpTopic(null, HELP_PAGE);

        return t;
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    public void writeProperties() {
        nodePanel.writeProperties();
    }

    /**
     * Constructor declaration
     *
     *
     * @see
     */
    public XMLCNodePropertyPage() {
        try {
            jbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Method declaration
     *
     *
     * @exception Exception
     *
     * @see
     */
    private void jbInit() throws Exception {
        this.setLayout(layoutMain);
        this.add(nodePanel, BorderLayout.CENTER);
    }

}
