/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id$
 */
package org.enhydra.kelp.jbuilder.wizard.wmlapp;

// Kelp imports
import org.enhydra.kelp.jbuilder.wizard.AbstractCodeGenWizard;
import org.enhydra.kelp.common.wizard.codegen.WirelessAppUtil;
import org.enhydra.kelp.common.wizard.codegen.CodeGenException;
import org.enhydra.kelp.common.swing.SwingUtil;
import org.enhydra.kelp.common.swing.IconSet;
import org.enhydra.kelp.KelpVersion;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterNodeFactory;
import org.enhydra.kelp.jbuilder.node.PrimeProject;
import org.enhydra.kelp.jbuilder.node.PrimeProject;
import org.enhydra.kelp.common.node.OtterFolderNode;

// JBuilder Foundation imports
import com.borland.jbuilder.node.JBProject;
import com.borland.primetime.wizard.Wizard;
import com.borland.primetime.wizard.WizardAction;
import com.borland.primetime.wizard.WizardHost;
import com.borland.primetime.wizard.WizardManager;
import com.borland.primetime.wizard.WizardPage;

// Standard imports
import javax.swing.JOptionPane;
import java.util.Vector;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%p
 */
public class WirelessAppWizard extends AbstractCodeGenWizard {
    private final String       HELP_TOPIC =
        "file:../kelp1.2/doc/kelp.html#appwizard";
    public static WizardAction WIZARD_EnhydraAppWizard =
        new WizardAction(WirelessAppUtil.getDisplayName(), 'X',
                         "Creates a new " + WirelessAppUtil.getDisplayName(),
                         IconSet.getSmallIcon(), IconSet.getLargeIcon(),
                         true, "Enhydra") {

        /**
         * Method declaration
         *
         *
         * @return
         *
         * @see
         */
        protected Wizard createWizard() {
            WirelessAppWizard wizard = new WirelessAppWizard();

            return wizard;
        }

        /**
         * Method declaration
         *
         *
         * @param source
         *
         * @see
         */
        public void update(Object source) {
            setEnabled(true);
        }

    };

    /**
     * Method declaration
     *
     *
     * @param majorVersion
     * @param minorVersion
     *
     * @see
     */
    public static void initOpenTool(byte majorVersion, byte minorVersion) {
        WizardManager.registerWizardAction(WIZARD_EnhydraAppWizard);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getDisplayName() {
        return WirelessAppUtil.getDisplayName();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getHelpTopic() {
        return HELP_TOPIC;
    }

    /**
     * Method declaration
     *
     *
     * @param host
     *
     * @return
     *
     * @see
     */
    public WizardPage invokeWizard(WizardHost host) {
         WizardPage page = null;
        OtterProject otterProject = null;
        WirelessAppUtil   util = null;

        otterProject = initProject(host);
        if (otterProject != null) {
          try {
            if (! WirelessAppUtil.isCodeGenAvailable()) {
              throw new CodeGenException("Enhydra Code Generator classes not found in IDE class path");
            }
            util = new WirelessAppUtil(otterProject);
            page = invokeWizard(host, util);
          } catch (CodeGenException e) {
            JOptionPane.showMessageDialog(
               host.getDialogParent(),
               e.getMessage(),
               WirelessAppUtil.getDisplayName() + " Wizard",
               JOptionPane.ERROR_MESSAGE);
          }
        }
        return page;
    }

    /**
     *
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String[] getSteps() {
        Vector steps = new Vector();
        String[] array = new String[0];
        if (KelpVersion.getEnhydraVersion().startsWith("4")) {
          steps.addElement(
             "Add a library to your project that contains j2ee.jar");
          if (System.getProperty("java.version").startsWith("1.3")) {
            steps.addElement(
             "Set the project JDK to: " +
             "&nbsp;&nbsp;<code>java JDK-1.2.2</code>.");
          }
        }
        steps.addElement(
            "From the menu select Wizards | XMLC Compiler and click on Compile "
            + "to compile the Welcome.wml file into a DOM class.");
        steps.addElement(
            "Make the project to compile the java files.");
        steps.addElement(
          "Deploy the project by selecting Wizards | Local Deployment.<BR>" +
          "Click on Deploy to process the configuration and descriptor templates.");
        steps.addElement(
            "Start the Enhydra server by running the project.");
        steps.addElement(
           "From your wireless device emulator connect to: <BR>" +
           "&nbsp;&nbsp;<code>http://localhost:9000/</code><BR>");
        array = (String[]) steps.toArray(array);
        steps.removeAllElements();
        return array;
    }

}
