/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */
package org.enhydra.kelp.jdeveloper.wizard;

// Kelp imports
import org.enhydra.kelp.common.wizard.codegen.CodeGenUtil;
import org.enhydra.kelp.common.wizard.codegen.CodeGenException;
import org.enhydra.kelp.common.swing.IconSet;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.jdeveloper.node.OracleProject;

// JDeveloper imports
import borland.jbuilder.addin.Addin;
import borland.jbuilder.addin.Context;
import borland.jbuilder.addin.Container;
import borland.jbuilder.addin.IdeEnvironment;
import borland.jbuilder.addin.Wizard;
import borland.jbuilder.addin.Project;
import borland.jbuilder.addin.VersionedAddin;

// Standard imports
import java.awt.Image;
import java.io.File;
import java.util.Properties;
import javax.swing.JOptionPane;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
abstract public class AbstractCodeGenWizard implements Wizard, VersionedAddin {
    private String     imageFileName = new String();
    private Image      bigImage;

    abstract public String getDisplayName();
    abstract public String[] getSteps();

    /**
     * Method declaration
     *
     *
     * @param ide
     *
     * @see
     */
    public void startup(IdeEnvironment ide) {
        if (imageFileName.length() > 0) {

            // bigImage = imageImageLoader.loadFromResource(imageFileName,
            // ide.getIdeFrame(), getClass());
        }
    }

    /**
     * For the VersionAddin Interface.
     */
    public String version() {
        return "2.0";
    }

    public boolean isJava2(IdeEnvironment ide) {
        boolean java2 = true;
        String ver = null;

        ver = System.getProperty("java.version");
        if (ver.startsWith("1.1")) {
          java2 = false;
          JOptionPane.showMessageDialog(ide.getIdeFrame(),
                                        "Kelp requires Java 2.\n" +
                                        "Found version: " + ver);
        }
        return java2;
    }


    /**
     * Method declaration
     *
     *
     * @param ide
     * @param context
     *
     * @see
     */
    public void invoke(IdeEnvironment ide, Context context,
          CodeGenUtil util) throws CodeGenException {
        Project nativeProject = null;
        int choice = CodeGenUtil.CHOICE_CANCEL;

        if (context != null) {
            nativeProject = context.getProject();
            if (nativeProject == null
                    || nativeProject.getType().equals(Container.ROOT_TYPE)) {
                nativeProject = null;
                JOptionPane.showMessageDialog(ide.getIdeFrame(),
                                              "Project not found. Please create a new project before "
                                              + " starting the " + getDisplayName() + ".");
            }
            if (nativeProject != null) {
                OtterProject otterProject = new OracleProject(nativeProject);
                nativeProject.BeginUpdate();
                nativeProject.setOutputDirectory(
                  otterProject.getCodeGenDefaultDestination() +
                  File.separator + "classes");
                nativeProject.save(true);
                nativeProject.EndUpdate();

                util.init(otterProject);
                util.getGenerator().setKelpSteps(getSteps());
                util.invokeWizard(ide.getIdeFrame());
                if (util.getChoice() == CodeGenUtil.CHOICE_FINISH) {
                   // Note steps should be added to replacement list!!!!
                    JOptionPane.showMessageDialog(ide.getIdeFrame(),
                                                  "The "
                                                  + getDisplayName()
                                                  + " Wizard added files to: \n"
                                                  + "     "
                                                  + nativeProject.getFilename()
                                                  + ".\n"
                                                  + "Refer to the readme.html file for the steps \n "
                                                  + "required to compile and run your new  \n"
                                                  + getDisplayName() + ".");
                    }

            }
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String toString() {
        return getDisplayName();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getName() {
        return getDisplayName();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getMenuLabel() {
        return null;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public Image getBigImage() {
        return IconSet.getLargeIcon().getImage();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public Image getSmallImage() {
        return IconSet.getSmallIcon().getImage();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public int getInfo() {
        return Wizard.REQUIRES_PROJECT | Wizard.PRODUCES_FILE;
    }
}
